@@ -22,39 +22,7 @@
 
 static int initialized = 0;
 
-#define ALIGNCHECK(mh)
-#define ALIGNCHECKK
-/* On compilers that support data alignment but not the automatic stack realignment.
-   We check for properly aligned stack before risking a crash because of badly compiled
-   client program. */
-#if (defined CCALIGN) && (defined NEED_ALIGNCHECK) && ((defined DEBUG) || (defined CHECK_ALIGN))
-
-/* Common building block. */
-#define ALIGNMAINPART \
-	/* minimum size of 16 bytes, not all compilers would align a smaller piece of data */ \
-	double ALIGNED(16) altest[2]; \
-	debug2("testing alignment, with %lu %% 16 = %lu", \
-		(unsigned long)altest, (unsigned long)((size_t)altest % 16)); \
-	if((size_t)altest % 16 != 0)
-
-#undef ALIGNCHECK
-#define ALIGNCHECK(mh) \
-	ALIGNMAINPART \
-	{ \
-		error("Stack variable is not aligned! Your combination of compiler/library is dangerous!"); \
-		if(mh != NULL) mh->err = MPG123_BAD_ALIGN; \
-\
-		return MPG123_ERR; \
-	}
-#undef ALIGNCHECKK
-#define ALIGNCHECKK \
-	ALIGNMAINPART \
-	{ \
-		error("Stack variable is not aligned! Your combination of compiler/library is dangerous!"); \
-		return MPG123_BAD_ALIGN; \
-	}
-
-#endif
+#include "aligncheck.h"
 
 #ifdef GAPLESS
 /*
@@ -322,11 +290,8 @@ int attribute_align_arg mpg123_par(mpg123_pars *mp, enum mpg123_parms key, long
 			mp->outscale = val == 0 ? fval : (double)val/SHORT_SCALE;
 		break;
 		case MPG123_TIMEOUT:
-#if (!defined (WIN32) || defined (__CYGWIN__))
 			mp->timeout = val >= 0 ? val : 0;
-#else
-			ret = MPG123_NO_TIMEOUT;
-#endif
+			/*ret = MPG123_NO_TIMEOUT;*/
 		break;
 		case MPG123_RESYNC_LIMIT:
 			mp->resync_limit = val;
@@ -510,6 +475,21 @@ int attribute_align_arg mpg123_open_fd(mpg123_handle *mh, int fd)
 	return open_stream(mh, NULL, fd);
 }
 
+int attribute_align_arg mpg123_open_handle(mpg123_handle *mh, void *iohandle)
+{
+	ALIGNCHECK(mh);
+	if(mh == NULL) return MPG123_ERR;
+
+	mpg123_close(mh);
+	frame_reset(mh);
+	if(mh->rdat.r_read_handle == NULL)
+	{
+		mh->err = MPG123_BAD_CUSTOM_IO;
+		return MPG123_ERR;
+	}
+	return open_stream_handle(mh, iohandle);
+}
+
 int attribute_align_arg mpg123_open_feed(mpg123_handle *mh)
 {
 	ALIGNCHECK(mh);
@@ -526,11 +506,27 @@ int attribute_align_arg mpg123_replace_reader( mpg123_handle *mh,
 {
 	ALIGNCHECK(mh);
 	if(mh == NULL) return MPG123_ERR;
+
+	mpg123_close(mh);
 	mh->rdat.r_read = r_read;
 	mh->rdat.r_lseek = r_lseek;
 	return MPG123_OK;
 }
 
+int attribute_align_arg mpg123_replace_reader_handle( mpg123_handle *mh,
+                           ssize_t (*r_read) (void*, void *, size_t),
+                           off_t   (*r_lseek)(void*, off_t, int),
+                           void    (*cleanup)(void*)  )
+{
+	ALIGNCHECK(mh);
+	if(mh == NULL) return MPG123_ERR;
+
+	mpg123_close(mh);
+	mh->rdat.r_read_handle = r_read;
+	mh->rdat.r_lseek_handle = r_lseek;
+	mh->rdat.cleanup_handle = cleanup;
+	return MPG123_OK;
+}
 
 int decode_update(mpg123_handle *mh)
 {
@@ -668,20 +664,22 @@ static int get_next_frame(mpg123_handle *mh)
 debug1("new format: %i", mh->new_format);
 
 		mh->decoder_change = 0;
-#ifdef GAPLESS
 		if(mh->fresh)
 		{
+#ifdef GAPLESS
 			int b=0;
 			/* Prepare offsets for gapless decoding. */
 			debug1("preparing gapless stuff with native rate %li", frame_freq(mh));
 			frame_gapless_realinit(mh);
 			frame_set_frameseek(mh, mh->num);
+#endif
 			mh->fresh = 0;
+#ifdef GAPLESS
 			/* Could this possibly happen? With a real big gapless offset... */
 			if(mh->num < mh->firstframe) b = get_next_frame(mh);
 			if(b < 0) return b; /* Could be error, need for more, new format... */
-		}
 #endif
+		}
 	}
 	return MPG123_OK;
 }
@@ -1153,7 +1151,7 @@ static int do_the_seek(mpg123_handle *mh)
 	if(mh->down_sample == 3)
 	{
 		ntom_set_ntom(mh, fnum);
-		debug3("fixed ntom for frame %"OFF_P" to %lu, num=%"OFF_P, fnum, mh->ntom_val[0], mh->num);
+		debug3("fixed ntom for frame %"OFF_P" to %lu, num=%"OFF_P, (off_p)fnum, mh->ntom_val[0], (off_p)mh->num);
 	}
 #endif
 	b = mh->rd->seek_frame(mh, fnum);
@@ -1340,10 +1338,13 @@ off_t attribute_align_arg mpg123_length(mpg123_handle *mh)
 	length = frame_ins2outs(mh, length);
 	debug1("mpg123_length: external sample length: %"OFF_P, (off_p)length);
 #ifdef GAPLESS
-	debug2("mpg123_length: begin_os = %"OFF_P", end_os = %"OFF_P, (off_p)mh->begin_os, (off_p)mh->end_os);
-	if(mh->end_os > 0 && length > mh->end_os) length = mh->end_os;
-	length -= mh->begin_os;
-	debug1("mpg123_length: after gapless correction: %"OFF_P, (off_p)length);
+	if(mh->p.flags & MPG123_GAPLESS)
+	{
+		debug2("mpg123_length: begin_os = %"OFF_P", end_os = %"OFF_P, (off_p)mh->begin_os, (off_p)mh->end_os);
+		if(mh->end_os > 0 && length > mh->end_os) length = mh->end_os;
+		length -= mh->begin_os;
+		debug1("mpg123_length: after gapless correction: %"OFF_P, (off_p)length);
+	}
 #endif
 	return length;
 }
@@ -1373,11 +1374,13 @@ int attribute_align_arg mpg123_scan(mpg123_handle *mh)
 	/* One frame must be there now. */
 	mh->track_frames = 1;
 	mh->track_samples = spf(mh); /* Internal samples. */
+	debug("TODO: We should disable gapless code when encountering inconsistent spf(mh)!");
 	while(read_frame(mh) == 1)
 	{
 		++mh->track_frames;
 		mh->track_samples += spf(mh);
 	}
+	debug2("Scanning yielded %"OFF_P" track samples, %"OFF_P" frames.", (off_p)mh->track_samples, (off_p)mh->track_frames);
 #ifdef GAPLESS
 	/* Also, think about usefulness of that extra value track_samples ... it could be used for consistency checking. */
 	frame_gapless_update(mh, mh->track_samples);
@@ -1632,6 +1635,8 @@ static const char *mpg123_error[] =
 	"Feature not in this build."
 	,"Some bad value has been provided."
 	,"Low-level seeking has failed (call to lseek(), usually)."
+	,"Custom I/O obviously not prepared."
+	,"Overflow in LFS (large file support) conversion."
 };
 
 const char* attribute_align_arg mpg123_plain_strerror(int errcode)
